@@ -14,6 +14,7 @@ from simmate.database.base_data_types import (
     Structure,
     table_column,
 )
+from simmate.visualization.plotting import MatplotlibFigure
 
 
 class BandStructure(DatabaseTable):
@@ -25,7 +26,20 @@ class BandStructure(DatabaseTable):
     class Meta:
         abstract = True
 
-    base_info = ["band_structure_data"]
+    exclude_from_summary = ["band_structure_data"]
+
+    archive_fields = ["band_structure_data"]
+
+    api_filters = dict(
+        nbands=["range"],
+        band_gap=["range"],
+        band_gap_direct=["range"],
+        is_gap_direct=["exact"],
+        energy_fermi=["range"],
+        conduction_band_minimum=["range"],
+        valence_band_maximum=["range"],
+        is_metal=["exact"],
+    )
 
     # kpt_path_type (setyawan_curtarolo, hinuma, latimer_munro)
     # Maybe set as an abstract property?
@@ -82,6 +96,14 @@ class BandStructure(DatabaseTable):
     # magnetic_ordering (Magnetic ordering of the calculation.)
     # equivalent_labels (Equivalent k-point labels in other k-path conventions)
 
+    def write_output_summary(self, directory: Path):
+        """
+        In addition to writing the normal VASP output summary, this also plots
+        the bandstructure to "band_structure.png"
+        """
+        super().write_output_summary(directory)
+        self.write_band_diagram_plot(directory=directory)
+
     @classmethod
     def _from_toolkit(
         cls,
@@ -117,23 +139,6 @@ class BandStructure(DatabaseTable):
         """
         return ToolkitBandStructure.from_dict(self.band_structure_data)
 
-    def get_bandstructure_plot(self):  # -> matplotlib figure
-        """
-        Plots the band structure using matplotlib
-        """
-
-        # NOTE: This method should be moved to a toolkit object
-
-        # DEV NOTE: Pymatgen only implements matplotlib for their band-structures
-        # at the moment, but there are two scripts location elsewhere that can
-        # outline how this can be done with Plotly:
-        # https://plotly.com/python/v3/ipython-notebooks/density-of-states/
-        # https://github.com/materialsproject/crystaltoolkit/blob/main/crystal_toolkit/components/bandstructure.py
-
-        bs_plotter = BSPlotter(self.to_toolkit_band_structure())
-        plot = bs_plotter.get_plot()
-        return plot
-
 
 class BandStructureCalc(Structure, BandStructure, Calculation):
     """
@@ -145,54 +150,36 @@ class BandStructureCalc(Structure, BandStructure, Calculation):
     class Meta:
         app_label = "workflows"
 
-    base_info = Structure.base_info + BandStructure.base_info + Calculation.base_info
-
-    def update_from_vasp_run(
-        self,
-        vasprun: Vasprun,
-        corrections: list,
-        directory: Path,
-    ):
-        """
-        Given a pymatgen VaspRun object, which is what's typically returned
-        from a simmate VaspWorkflow.run() call, this will update the database entry
-        with the results.
-        """
-
-        # All data analysis is done via a BandStructure object, so we convert
-        # the vasprun object to that first.
-        band_structure = vasprun.get_band_structure(line_mode=True)
-
-        # Take our band_structure and expand its data for the rest of the columns.
-        new_kwargs = BandStructure.from_toolkit(
-            band_structure=band_structure,
-            as_dict=True,
-        )
-        for key, value in new_kwargs.items():
-            setattr(self, key, value)
-
-        # lastly, we also want to save the corrections made and directory it ran in
-        self.corrections = corrections
-        self.directory = directory
-
-        # Now we have the relaxation data all loaded and can save it to the database
-        self.save()
-
     @classmethod
-    def from_directory(cls, directory: Path):
-        """
-        Creates a new database entry from a directory that holds band-structure
-        results. For now, this assumes the directory holds vasp output files.
-        """
+    def from_vasp_run(cls, vasprun: Vasprun, as_dict: bool = False):
 
-        # I assume the directory is from a vasp calculation, but I need to update
-        # this when I begin adding new calculators.
-        vasprun_filename = directory / "vasprun.xml"
-        vasprun = Vasprun(vasprun_filename)
         band_structure = vasprun.get_band_structure(line_mode=True)
         band_structure_db = cls.from_toolkit(
             structure=vasprun.structures[0],
             band_structure=band_structure,
+            as_dict=as_dict,
         )
-        band_structure_db.save()
+        if not as_dict:
+            band_structure_db.save()
         return band_structure_db
+
+
+class BandDiagram(MatplotlibFigure):
+    def get_plot(result: BandStructure):
+
+        # NOTE: This method should be moved to a toolkit object
+
+        # DEV NOTE: Pymatgen only implements matplotlib for their band-structures
+        # at the moment, but there are two scripts location elsewhere that can
+        # outline how this can be done with Plotly:
+        # https://plotly.com/python/v3/ipython-notebooks/density-of-states/
+        # https://github.com/materialsproject/crystaltoolkit/blob/main/crystal_toolkit/components/bandstructure.py
+
+        bs_plotter = BSPlotter(result.to_toolkit_band_structure())
+        plot = bs_plotter.get_plot()
+        return plot
+
+
+# register all plotting methods to the database table
+for _plot in [BandDiagram]:
+    _plot.register_to_class(BandStructure)
