@@ -12,6 +12,7 @@ from simmate.database.base_data_types import (
     Structure,
     table_column,
 )
+from simmate.visualization.plotting import MatplotlibFigure
 
 
 class DensityofStates(DatabaseTable):
@@ -23,7 +24,16 @@ class DensityofStates(DatabaseTable):
     class Meta:
         abstract = True
 
-    base_info = ["density_of_states_data"]
+    exclude_from_summary = ["density_of_states_data"]
+
+    archive_fields = ["density_of_states_data"]
+
+    api_filters = dict(
+        band_gap=["range"],
+        energy_fermi=["range"],
+        conduction_band_minimum=["range"],
+        valence_band_maximum=["range"],
+    )
 
     # !!! Consider breaking down data into...
     #   total
@@ -60,6 +70,15 @@ class DensityofStates(DatabaseTable):
     # spin_polarization (float "Spin polarization at the fermi level.")
     # magnetic_ordering (Magnetic ordering of the calculation.)
 
+    def write_output_summary(self, directory: Path):
+        """
+        In addition to writing the normal VASP output summary, this also plots
+        the DOS to "density_of_states.png"
+        """
+
+        super().write_output_summary(directory)
+        self.write_dos_diagram_plot(directory=directory)
+
     @classmethod
     def _from_toolkit(
         cls,
@@ -71,10 +90,10 @@ class DensityofStates(DatabaseTable):
         data = (
             dict(
                 density_of_states_data=density_of_states.as_dict(),
-                band_gap=density_of_states.get_gap(),
+                band_gap=float(density_of_states.get_gap()),
                 energy_fermi=density_of_states.efermi,
-                conduction_band_minimum=density_of_states.get_cbm_vbm()[0],
-                valence_band_maximum=density_of_states.get_cbm_vbm()[1],
+                conduction_band_minimum=float(density_of_states.get_cbm_vbm()[0]),
+                valence_band_maximum=float(density_of_states.get_cbm_vbm()[1]),
             )
             if density_of_states
             else {}
@@ -91,10 +110,32 @@ class DensityofStates(DatabaseTable):
         """
         return CompleteDos.from_dict(self.density_of_states_data)
 
-    def get_densityofstates_plot(self):
-        """
-        Plots the density of states using matplotlib
-        """
+
+class DensityofStatesCalc(Structure, DensityofStates, Calculation):
+    """
+    Holds Structure, DensityofStates, and Calculation information. Density of state
+    workflows are common in materials science, so this table defines the most
+    common data that results from such workflow calculations.
+    """
+
+    class Meta:
+        app_label = "workflows"
+
+    @classmethod
+    def from_vasp_run(cls, vasprun: Vasprun, as_dict: bool = False):
+
+        density_of_states_db = cls.from_toolkit(
+            structure=vasprun.structures[0],
+            density_of_states=vasprun.complete_dos,
+            as_dict=as_dict,
+        )
+        if not as_dict:
+            density_of_states_db.save()
+        return density_of_states_db
+
+
+class DosDiagram(MatplotlibFigure):
+    def get_plot(result: DensityofStates):
 
         # NOTE: This method should be moved to a toolkit object
 
@@ -105,7 +146,7 @@ class DensityofStates(DatabaseTable):
         # https://github.com/materialsproject/crystaltoolkit/blob/main/crystal_toolkit/components/bandstructure.py
 
         plotter = DosPlotter()
-        complete_dos = self.to_toolkit_density_of_states()
+        complete_dos = result.to_toolkit_density_of_states()
 
         # Add the total density of States
         plotter.add_dos("Total DOS", complete_dos)
@@ -124,63 +165,6 @@ class DensityofStates(DatabaseTable):
         return plot
 
 
-class DensityofStatesCalc(Structure, DensityofStates, Calculation):
-    """
-    Holds Structure, DensityofStates, and Calculation information. Density of state
-    workflows are common in materials science, so this table defines the most
-    common data that results from such workflow calculations.
-    """
-
-    class Meta:
-        app_label = "workflows"
-
-    base_info = Structure.base_info + DensityofStates.base_info + Calculation.base_info
-
-    def update_from_vasp_run(
-        self, vasprun: Vasprun, corrections: list, directory: Path
-    ):
-        """
-        Given a pymatgen VaspRun object, which is what's typically returned
-        from a simmate VaspWorkflow.run() call, this will update the database entry
-        with the results.
-        """
-
-        # Takes a pymatgen VaspRun object, which is what's typically returned
-        # from a simmate VaspWorkflow.run() call.
-
-        # All data analysis is done via a CompleteDOS object, so we convert
-        # the vasprun object to that first.
-        density_of_states = vasprun.complete_dos
-
-        # Take our dos and expand its data for the rest of the columns.
-        new_kwargs = DensityofStates.from_toolkit(
-            density_of_states=density_of_states,
-            as_dict=True,
-        )
-        for key, value in new_kwargs.items():
-            setattr(self, key, value)
-
-        # lastly, we also want to save the corrections made and directory it ran in
-        self.corrections = corrections
-        self.directory = directory
-
-        # Now we have the relaxation data all loaded and can save it to the database
-        self.save()
-
-    @classmethod
-    def from_directory(cls, directory: Path):
-        """
-        Creates a new database entry from a directory that holds band-structure
-        results. For now, this assumes the directory holds vasp output files.
-        """
-
-        # I assume the directory is from a vasp calculation, but I need to update
-        # this when I begin adding new calculators.
-        vasprun_filename = directory / "vasprun.xml"
-        vasprun = Vasprun(vasprun_filename)
-        density_of_states_db = cls.from_toolkit(
-            structure=vasprun.structures[0],
-            density_of_states=vasprun.complete_dos,
-        )
-        density_of_states_db.save()
-        return density_of_states_db
+# register all plotting methods to the database table
+for _plot in [DosDiagram]:
+    _plot.register_to_class(DensityofStates)
