@@ -13,54 +13,8 @@ This module helps you create tables to results from structure relaxations
 (aka geometry optimizations). This will store all ionic steps and the forces/stress
 associated with each step.
 
-When creating new tables for Relaxations, you should use the `Relaxation.create_subclasses` method, which helps remove all the 
-boilerplate code needed. For Django users, it may be tricky to understand what's
-happening behind the scenes, so here's an example:
-    
-These two lines...
-
-``` python 
-from simmate.database.base_data_types import Relaxation
-
-ExampleRelaxation, ExampleIonicStep = Relaxation.create_subclasses(
-    "Example",
-    module=__name__,
-)
-```
-
-... do exactly the same thing as all of these lines...
-
-``` python
-from simmate.database.base_data_types import table_column
-from simmate.database.base_data_types import IonicStep, Relaxation
-
-class ExampleIonicStep(IonicStep):
-    relaxation = table_column.ForeignKey(
-        "ExampleRelaxation",  # in quotes becuase this is defined below
-        on_delete=table_column.CASCADE,
-        related_name="structures",
-    )
-
-class ExampleRelaxation(Relaxation):
-    structure_start = table_column.OneToOneField(
-        ExampleIonicStep,
-        on_delete=table_column.CASCADE,
-        related_name="relaxations_as_start",
-        blank=True,
-        null=True,
-    )
-    structure_final = table_column.OneToOneField(
-        ExampleIonicStep,
-        on_delete=table_column.CASCADE,
-        related_name="relaxations_as_final",
-        blank=True,
-        null=True,
-    )
-```
-
 Note there are two tables involved. One stores all of the ionic steps, and the
 other connects all ionic steps to a specific calculation and result.
-
 """
 
 from pathlib import Path
@@ -76,6 +30,7 @@ from simmate.database.base_data_types import (
     Thermodynamics,
     table_column,
 )
+from simmate.visualization.plotting import PlotlyFigure
 
 
 class Relaxation(Structure, Thermodynamics, Calculation):
@@ -94,16 +49,23 @@ class Relaxation(Structure, Thermodynamics, Calculation):
     class Meta:
         app_label = "workflows"
 
-    base_info = (
-        [
-            "band_gap",
-            "is_gap_direct",
-            "energy_fermi",
-            "conduction_band_minimum",
-            "valence_band_maximum",
-        ]
-        + Structure.base_info
-        + Calculation.base_info
+    exclude_from_summary = ["structure_start", "structure_final"]
+
+    archive_fields = [
+        "band_gap",
+        "is_gap_direct",
+        "energy_fermi",
+        "conduction_band_minimum",
+        "valence_band_maximum",
+    ]
+
+    api_filters = dict(
+        volume_change=["range"],
+        band_gap=["exact", "range"],
+        is_gap_direct=["exact"],
+        energy_fermi=["range"],
+        conduction_band_minimum=["range"],
+        valence_band_maximum=["range"],
     )
 
     # OPTIMIZE: should I include this electronic data?
@@ -171,13 +133,9 @@ class Relaxation(Structure, Thermodynamics, Calculation):
         null=True,
     )
 
-    @classmethod
-    def from_directory(cls, directory: Path):
-        # I assume the directory is from a vasp calculation, but I need to update
-        # this when I begin adding new calculators.
-        vasprun_filename = directory / "vasprun.xml"
-        vasprun = Vasprun(vasprun_filename)
-        return cls.from_vasp_run(vasprun)
+    def write_output_summary(self, directory: Path):
+        super().write_output_summary(directory)
+        self.write_relaxation_convergence_plot(directory=directory)
 
     @classmethod
     def from_vasp_run(cls, vasprun: Vasprun):
@@ -188,27 +146,29 @@ class Relaxation(Structure, Thermodynamics, Calculation):
         # Note, the information does not matter at this point because it will be
         # populated below
         relaxation = cls.from_toolkit(structure=vasprun.structures[-1])
-        # TODO: need to pull run_id from metadata file.
 
         # Now we have the relaxation data all loaded and can save it to the database
         relaxation.save()
 
-        # Save the rest of the results using the update method from this class
-        relaxation.update_from_vasp_run(
-            vasprun=vasprun,
-            corrections=[],
-            directory=None,
-        )
-        # TODO: load corrections/directory from the metadata and corrections files.
+        # and we can populate the rest of the tables as if its the workup
+        relaxation.update_from_vasprun(vasprun)
 
-        return relaxation
+    def update_from_directory(self, directory: Path):
 
-    def update_from_vasp_run(
-        self,
-        vasprun: Vasprun,
-        corrections: list,
-        directory: Path,
-    ):
+        # check if we have a VASP directory
+        vasprun_filename = directory / "vasprun.xml"
+        if not vasprun_filename.exists():
+            # raise Exception(
+            #     "Only VASP output directories are supported at the moment"
+            # )
+            return  # just exit
+
+        from simmate.calculators.vasp.outputs import Vasprun
+
+        vasprun = Vasprun.from_directory(directory)
+        self.update_from_vasp_run(vasprun)
+
+    def update_from_vasp_run(self, vasprun: Vasprun):
         """
         Given a Vasprun object from a finished relaxation, this will update the
         Relaxation table entry and the corresponding IonicStep entries.
@@ -273,15 +233,81 @@ class Relaxation(Structure, Thermodynamics, Calculation):
             energy_fermi=data.get("efermi"),
             conduction_band_minimum=data.get("cbm"),
             valence_band_maximum=data.get("vbm"),
-            # lastly, we also want to save the corrections made and directory it ran in
-            corrections=corrections,
-            directory=directory,
         )
 
-    def get_convergence_plot(self):
+
+class IonicStep(Structure, Thermodynamics, Forces):
+    """
+    This database table that holds the data for each ionic step of a relaxation.
+
+    An ionic step can be viewed as a static energy calculation, but we keep these
+    results separate because:
+
+        1. Pulay stress can make these energies/forces inaccurate
+        2. These results each have an associated Relaxation and ionic step number
+
+    You will likely never access this table directly. Instead, data is better
+    accessed through the `structures` attribute on appropiate Relaxation table.
+
+    For example:
+
+    ``` python
+    from simmate.database import connect
+    from simmate.database.workflow_results import MITRelaxation
+
+    # grab your desired relaxation
+    relax = MITRelaxation.objects.get(id=1)
+
+    # grab the associated ionic steps
+    ionic_steps = relax.structures.all()
+    ```
+
+    Note: we assume that only converged data is being stored! So there is no
+    "converged_electronic" column here. ErrorHandlers and workups should
+    ensure convergence.
+    """
+
+    class Meta:
+        app_label = "workflows"
+
+    archive_fields = ["number"]
+
+    api_filters = dict(
+        numer=["range"],
+    )
+
+    number = table_column.IntegerField()
+    """
+    The ionic step number within the relaxation. This starts counting from 0.
+    """
+
+    """ Relationships """
+    # each of these will map to a Relaxation, so you should typically access this
+    # data through that class. The exception to this is when you want all ionic
+    # steps accross many relaxations for a machine learning input.
+    # These relationship can be found via...
+    #   relaxations
+    #   relaxations_as_start
+    #   relaxations_as_final
+
+    # All structures in this table come from relaxation calculations, where
+    # there can be many structures (one for each ionic steps) linked to a
+    # single relaxation. This means the start structure, end structure, and
+    # those structure in-between are stored together here.
+    # Therefore, there's just a simple column stating which relaxation it
+    # belongs to.
+    relaxation = table_column.ForeignKey(
+        "Relaxation",
+        on_delete=table_column.CASCADE,
+        related_name="structures",
+    )
+
+
+class RelaxationConvergence(PlotlyFigure):
+    def get_plot(relaxation: Relaxation):
 
         # Grab the calculation's structure and convert it to a dataframe
-        structures_dataframe = self.structures.order_by("number").to_dataframe()
+        structures_dataframe = relaxation.structures.order_by("number").to_dataframe()
 
         # We will be making a figure that consists of 3 stacked subplots that
         # all share the x-axis of ionic_step_number
@@ -332,83 +358,9 @@ class Relaxation(Structure, Thermodynamics, Calculation):
             yaxis3_title="Lattice Stress",
         )
 
-        # we return the figure object for the user
         return figure
 
-    def view_convergence_plot(self):
-        figure = self.get_convergence_plot()
-        figure.show(renderer="browser")
 
-    def get_convergence_plot_html(self):
-        # Make the convergence figure and convert it to an html div
-        figure_convergence = self.get_convergence_plot()
-        figure_convergence_html = figure_convergence.to_html(
-            full_html=False,
-            include_plotlyjs=False,
-        )
-        return figure_convergence_html
-
-
-class IonicStep(Structure, Thermodynamics, Forces):
-    """
-    This database table that holds the data for each ionic step of a relaxation.
-
-    An ionic step can be viewed as a static energy calculation, but we keep these
-    results separate because:
-
-        1. Pulay stress can make these energies/forces inaccurate
-        2. These results each have an associated Relaxation and ionic step number
-
-    You will likely never access this table directly. Instead, data is better
-    accessed through the `structures` attribute on appropiate Relaxation table.
-
-    For example:
-
-    ``` python
-    from simmate.database import connect
-    from simmate.database.workflow_results import MITRelaxation
-
-    # grab your desired relaxation
-    relax = MITRelaxation.objects.get(id=1)
-
-    # grab the associated ionic steps
-    ionic_steps = relax.structures.all()
-    ```
-
-    Note: we assume that only converged data is being stored! So there is no
-    "converged_electronic" column here. ErrorHandlers and workups should
-    ensure convergence.
-    """
-
-    class Meta:
-        app_label = "workflows"
-
-    base_info = (
-        ["number"] + Structure.base_info + Thermodynamics.base_info + Forces.base_info
-    )
-
-    number = table_column.IntegerField()
-    """
-    The ionic step number within the relaxation. This starts counting from 0.
-    """
-
-    """ Relationships """
-    # each of these will map to a Relaxation, so you should typically access this
-    # data through that class. The exception to this is when you want all ionic
-    # steps accross many relaxations for a machine learning input.
-    # These relationship can be found via...
-    #   relaxations
-    #   relaxations_as_start
-    #   relaxations_as_final
-
-    # All structures in this table come from relaxation calculations, where
-    # there can be many structures (one for each ionic steps) linked to a
-    # single relaxation. This means the start structure, end structure, and
-    # those structure in-between are stored together here.
-    # Therefore, there's just a simple column stating which relaxation it
-    # belongs to.
-    relaxation = table_column.ForeignKey(
-        "Relaxation",
-        on_delete=table_column.CASCADE,
-        related_name="structures",
-    )
+# register all plotting methods to the database table
+for _plot in [RelaxationConvergence]:
+    _plot.register_to_class(Relaxation)
