@@ -91,7 +91,7 @@ class S3Workflow(Workflow):
         command: str = None,
         is_restart: bool = False,
         **kwargs,
-    ):
+    ) -> dict:
         """
          Runs the entire staged task (setup, execution, workup), which includes
          supervising during execution.
@@ -180,13 +180,22 @@ class S3Workflow(Workflow):
 
         # run the workup stage of the task. This is where the data/info is pulled
         # out from the calculation and is thus our "result".
-        result = cls.workup(directory=directory)
+        extra_results = cls.workup(directory=directory) or {}
+
+        # Make sure the user is returning a compatible result from the workup
+        # method.
+        if not isinstance(extra_results, dict):
+            raise Exception(
+                "When defining a custom `workup` method, you must return a dictionary "
+                "(or None). This is so `corrections` can be added to your dictionary "
+                "and also the dictionary is used to update database columns when "
+                " `use_database=True`"
+            )
 
         # Return our final information as a dictionary
         result = {
-            "result": result,
             "corrections": corrections,
-            "directory": directory,
+            **extra_results,
         }
 
         return result
